@@ -15,11 +15,11 @@ import logging
 import os
 import pprint
 import re
+import requests
 import sys
 
-import requests
-
 from cmd import Cmd
+from datetime import datetime
 from ibm_watson_machine_learning.wml_client_error import ApiRequestFailure
 
 import dowml
@@ -63,9 +63,9 @@ job id, but none is specified, the last one is used.
 
 ''')
 
-    def __init__(self, wml_cred_file=None, space_id=None, url=None, region=None):
+    def __init__(self, wml_cred_file=None, api_key=None, space_id=None, url=None, region=None):
         super().__init__()
-        self.lib = DOWMLLib(wml_cred_file, space_id, url=url, region=region)
+        self.lib = DOWMLLib(wml_cred_file, api_key=api_key, space_id=space_id, url=url, region=region)
         self.jobs = []
         self.last_job_id = None
 
@@ -411,7 +411,7 @@ Uses current job if not specified."""
 Stops the job specified.
 job is either a job number or a job id. Uses current job if not specified."""
         job_id = self._get_and_remember_job_id(job_id)
-        self.lib.delete_job(job_id, False)
+        self.lib.cancel_job(job_id)
 
     @staticmethod
     def do_shell(command):
@@ -481,7 +481,7 @@ def main_loop(instance, commands, prompt_at_the_end):
             instance.intro = ''
 
 
-# We will mock the 'requests' function that's used by APIClient
+# We will patch the 'requests' function that's used by APIClient
 # So first we save the original function
 orig_requests_session_send = requests.Session.send
 
@@ -491,10 +491,11 @@ def mocked_requests_session_send(*arguments, **kwargs):
     session, prepared_request = arguments
     method = prepared_request.method
     url = prepared_request.url
-    #       2021-02-17 16:59:39,710
-    print(f'           {method} {url}')
+    dt_iso = datetime.now().isoformat(sep=' ', timespec='milliseconds')
+    print(f'{dt_iso} {method} {url}')
     resp = orig_requests_session_send(*arguments, **kwargs)
-    # print(f'                        {resp.status_code}')
+    dt_iso = datetime.now().isoformat(sep=' ', timespec='milliseconds')
+    print(f'{dt_iso} {resp.status_code}')
     return resp
 
 
@@ -530,6 +531,10 @@ def interactive():
                              f'the one specified in the credentials under the '
                              f'\'{_CredentialsProvider.URL}\' key, if any. '
                              f'Incompatible with --region argument.')
+    parser.add_argument('--api-key', '-k', default=None,
+                        help=f'API key to use to connect to WML. Takes precedence over '
+                             f'the one specified in the credentials under the '
+                             f'\'{_CredentialsProvider.APIKEY}\' key, if any.')
     regions = list(_CredentialsProvider.REGION_TO_URL.keys())
     parser.add_argument('--region', '-r', default=None,
                         help=f'Region to use for the Machine Learning service. Takes precedence over '
@@ -561,7 +566,8 @@ def interactive():
         # logging.getLogger('ibm_boto3').setLevel(logging.DEBUG)
 
     try:
-        instance = DOWMLInteractive(args.wml_cred_file, args.space, url=args.url, region=args.region)
+        instance = DOWMLInteractive(args.wml_cred_file, api_key=args.api_key,
+                                    space_id=args.space, url=args.url, region=args.region)
         main_loop(instance, args.commands, args.input)
     except InvalidCredentials as e:
         logging.getLogger().error(f"ERROR: {e.__doc__}", exc_info=True)
