@@ -84,6 +84,7 @@ def _new_params():
     global _the_old_params
     global _the_filter
     # Use the original code and get its output
+    # noinspection PyCallingNonCallable
     result = _the_old_params()
     # Add the filter, if one is required
     if _the_filter:
@@ -122,7 +123,7 @@ class _CredentialsProvider:
         'jp-tok': 'https://jp-tok.ml.cloud.ibm.com',
     }
 
-    def __init__(self, wml_credentials_file=None, wml_credentials_str=None, url=None, region=None):
+    def __init__(self, wml_credentials_file=None, api_key=None, wml_credentials_str=None, url=None, region=None):
         self._logger = logging.getLogger(self.__class__.__name__)
 
         if wml_credentials_str is None:
@@ -130,21 +131,37 @@ class _CredentialsProvider:
                 wml_credentials_str = self._read_wml_credentials_from_file(wml_credentials_file)
             else:
                 wml_credentials_str = self._read_wml_credentials_from_env()
-            self._logger.debug('Found credential string.')
-        self.credentials = self.check_credentials(wml_credentials_str, url=url, region=region)
+            if wml_credentials_str:
+                self._logger.debug('Found credential string.')
+        self.credentials = self.check_credentials(wml_credentials_str, api_key=api_key, url=url, region=region)
 
     def usage(self):
         print(f'${self.ENVIRONMENT_VARIABLE_NAME} should contain credentials as a Python dict of the form:')
         print(f'  {{\'{self.APIKEY}\': \'<apikey>\', \'{self.URL}\': \'https://us-south.ml.cloud.ibm.com\'}}')
         print(f'Or set ${self.ENVIRONMENT_VARIABLE_NAME_FILE} to the path to a file containing the same information.')
 
-    def check_credentials(self, wml_cred_str, url, region):
-        assert type(wml_cred_str) is str
-        if not wml_cred_str:
-            raise InvalidCredentials('WML credentials must not be an empty string.')
-        wml_credentials = ast.literal_eval(wml_cred_str)
-        assert type(wml_credentials) is dict
-        assert (self.APIKEY in wml_credentials or self.TOKEN in wml_credentials)
+    def check_credentials(self, wml_cred_str, api_key, url, region):
+        wml_credentials = None
+        if wml_cred_str is not None:
+            assert type(wml_cred_str) is str
+            if not wml_cred_str:
+                raise InvalidCredentials('WML credentials must not be an empty string.')
+            wml_credentials = ast.literal_eval(wml_cred_str)
+            assert type(wml_credentials) is dict
+        if not wml_credentials:
+            # If we don't find a credentials string through the environment, we will
+            # assume that the parameters are enough to build one.
+            wml_credentials = {}
+            if api_key:
+                wml_credentials[self.APIKEY] = api_key
+            if url:
+                wml_credentials[self.URL] = url
+            if region:
+                wml_credentials[self.REGION] = region
+        if self.APIKEY not in wml_credentials and self.TOKEN not in wml_credentials:
+            raise InvalidCredentials('API key (or token) must be specified.')
+        if self.URL not in wml_credentials and self.REGION not in wml_credentials:
+            raise InvalidCredentials('URL or region must be specified (but not both).')
         if self.APIKEY in wml_credentials:
             assert type(wml_credentials[self.APIKEY]) is str
         else:
@@ -186,14 +203,12 @@ class _CredentialsProvider:
         var_name = self.ENVIRONMENT_VARIABLE_NAME
         var_file_name = self.ENVIRONMENT_VARIABLE_NAME_FILE
         self._logger.debug(f'Looking for credentials in environment variable {var_name}...')
+        wml_cred_str = None
         if var_name in os.environ:
             wml_cred_str = os.environ[var_name]
         elif var_file_name in os.environ:
             self._logger.debug(f'Looking for credentials file name in environment variable {var_file_name}...')
             wml_cred_str = self._read_wml_credentials_from_file(os.environ[var_file_name])
-        else:
-            raise InvalidCredentials(f'Environment variables ${var_name} or ${var_file_name} not found.')
-
         return wml_cred_str
 
     def _read_wml_credentials_from_file(self, file):
@@ -237,6 +252,7 @@ class DOWMLLib:
     DEPLOYMENT_NAME = f'{DOWML_PREFIX}-deployment'
 
     def __init__(self, wml_credentials_file=None,
+                 api_key=None,
                  space_id=None,
                  url=None,
                  region=None,
@@ -271,7 +287,7 @@ class DOWMLLib:
 
         self._logger = logging.getLogger(self.__class__.__name__)
 
-        cred_provider = _CredentialsProvider(wml_credentials_file, url=url, region=region)
+        cred_provider = _CredentialsProvider(wml_credentials_file, api_key=api_key, url=url, region=region)
         wml_credentials = cred_provider.credentials
 
         # A space name in the credentials changes the default
@@ -381,10 +397,6 @@ class DOWMLLib:
                 else:
                     if not first_try:
                         self._logger.warning('Clearing the cache didn\'t help...')
-                    else:
-                        # This is not the error we expected
-                        pass
-                    # either way, there's nothing we can do on this one
                     raise
 
     def client_data_asset_download(self, asset_id, filename):
@@ -708,20 +720,25 @@ class DOWMLLib:
         client = self._get_or_make_client()
         wml_url = self._wml_credentials['url']
         # We don't want to (try to) delete the WS run if we only cancel the job
+        # Instead, we skip everything until calling deployments.delete_job(True)
         everything_ok_so_far = hard
         ws_url = client.PLATFORM_URLS_MAP.get(wml_url)
-        if not ws_url:
+        if everything_ok_so_far and not ws_url:
+            self._logger.error(f'Unknown Watson Studio URL for WML URL {wml_url}.')
             everything_ok_so_far = False
-        url = None  # Only required to silence the editor's warning
         if everything_ok_so_far:
             try:
                 platform_job_id = job_details['entity']['platform_job']['job_id']
                 platform_run_id = job_details['entity']['platform_job']['run_id']
                 url = f'{ws_url}/v2/jobs/{platform_job_id}/runs/{platform_run_id}?space_id={self.space_id}'
             except KeyError:
+                self._logger.error('Watson Studio job id or run id not found in WML job details.')
                 everything_ok_so_far = False
         if everything_ok_so_far:
+            # noinspection PyUnboundLocalVariable
+            self._logger.debug(f'Trying to delete run {platform_run_id} of Watson Studio job {platform_job_id}...')
             # noinspection PyProtectedMember
+            # noinspection PyUnboundLocalVariable
             r = requests.delete(url, headers={'Authorization': f'Bearer {client.service_instance._get_token()}',
                                               'Content-Type': 'application/json',
                                               'cache-control': 'no-cache'})
@@ -735,18 +752,29 @@ class DOWMLLib:
             #   we just wanted to cancel the job, so there's nothing to warn against
             client.deployments.delete_job(job_id, hard)
 
-    def delete_job(self, job_id, hard=False):
+    def delete_job(self, job_id, hard=True):
         """Delete the given job.
 
         :param job_id: the job to be deleted
-        :param hard: if False, cancel the job. If true, delete it completely
+        :param hard: deprecated. Use cancel_job instead of passing False
         """
-        job_details = None
-        if hard:
-            job_details = self.get_job_details(job_id, with_contents='names')
-            self._delete_data_assets(job_details)
-        self._logger.debug(f'Deleting job {job_id}...')
-        self._client_deployments_delete_job(job_id, hard, job_details)
+        if not hard:
+            self.cancel_job(job_id)
+            return
+        self._logger.debug(f'Deleting data assets (if any) for job {job_id}...')
+        job_details = self.get_job_details(job_id, with_contents='names')
+        self._delete_data_assets(job_details)
+        self._logger.debug(f'Done. Deleting job {job_id}...')
+        self._client_deployments_delete_job(job_id, True, job_details)
+        self._logger.debug('Done.')
+
+    def cancel_job(self, job_id):
+        """Cancel the given job.
+
+        :param job_id: the job to be canceled
+        """
+        self._logger.debug(f'Canceling job {job_id}...')
+        self._client_deployments_delete_job(job_id, False, None)
         self._logger.debug('Done.')
 
     def decode_log(self, output):
@@ -876,7 +904,6 @@ class DOWMLLib:
                             print('')
                             # We are joining the lines in the activity with a CR,
                             # only to remove them if they were already included...
-                            # FIXME: what a waste!
                             act = '\n'.join(activity)
                             act = self.remove_empty_lines(act)
                             print(act)
